#ifndef MSP_GLTK_BUTTON_H_
#define MSP_GLTK_BUTTON_H_

#include <sigc++/sigc++.h>
#include <msp/gl/texture2d.h>
#include "text.h"
#include "widget.h"

namespace Msp {
namespace GLtk {

/**
A clickable button widget.  Buttons can have a text label, which is displayed
in a spacial part "text".
*/
class Button: public Widget
{
public:
	class Loader: public DataFile::DerivedObjectLoader<Button, Widget::Loader>
	{
	public:
		Loader(Button &);
	private:
		void text(const std::string &);
	};

	sigc::signal<void> signal_clicked;

private:
	Text text;
	const GL::Texture2D *icon;
	bool pressed;

public:
	Button(const std::string & = std::string());

	virtual const char *get_class() const { return "button"; }

	virtual void autosize();

	void set_text(const std::string &);
	void set_icon(const GL::Texture2D *);

private:
	virtual void rebuild_special(const Part &);

public:
	virtual void button_press(int, int, unsigned);
	virtual void button_release(int, int, unsigned);
	virtual void pointer_motion(int, int);
private:
	virtual void on_style_change();
};

} // namespace GLtk
} // namespace Msp

#endif
