#include "label.h"
#include "part.h"
#include "style.h"

using namespace std;

namespace Msp {
namespace GLtk {

Label::Label(const string &t)
{
	focusable = false;
	set_text(t);
}

void Label::autosize()
{
	if(!style)
		return;

	Widget::autosize();

	if(const Part *text_part = style->get_part("text"))
	{
		const Sides &margin = text_part->get_margin();
		geom.w = max(geom.w, text.get_width()+margin.left+margin.right);
		geom.h = max(geom.h, text.get_height()+margin.top+margin.bottom);
	}

	rebuild();
}

void Label::set_text(const string &t)
{
	text = t;
	signal_autosize_changed.emit();
	rebuild();
}

void Label::rebuild_special(const Part &part)
{
	if(part.get_name()=="text")
		text.build(part, geom, part_cache);
}

void Label::on_style_change()
{
	text.set_style(style);
}


Label::Loader::Loader(Label &l):
	DataFile::DerivedObjectLoader<Label, Widget::Loader>(l)
{
	add("text", &Loader::text);
}

void Label::Loader::text(const string &t)
{
	obj.text = t;
}

} // namespace GLtk
} // namespace Msp
