#ifndef MSP_GLTK_LIST_H_
#define MSP_GLTK_LIST_H_

#include <sigc++/signal.h>
#include "container.h"
#include "label.h"
#include "listdata.h"
#include "vslider.h"

namespace Msp {
namespace GLtk {

/**
Shows a list of items, allowing the user to select one.  A slider is included
to allow scrolling through a long list.
*/
class List: virtual public Widget, private Container
{
public:
	class Loader: public DataFile::DerivedObjectLoader<List, Widget::Loader>
	{
	public:
		Loader(List &);
	private:
		void item(const std::string &);
	};

private:
	/// This exists to make disconnecting signals easier
	class DataObserver: public sigc::trackable
	{
	private:
		List &list;

	public:
		DataObserver(List &);

		void item_added(unsigned);
		void item_removed(unsigned);
		void cleared();
		void refresh_strings();
	};

protected:
	class Item: public Container
	{
	public:
		virtual const char *get_class() const { return "listitem"; }

		virtual void autosize();
		void set_active(bool);

		virtual void render_special(const Part &, GL::Renderer &) const;
	};

public:
	class BasicItem: public Item
	{
	private:
		Label label;

	public:
		BasicItem(const std::string &);

	private:
		virtual void on_style_change();
	};

public:
	sigc::signal<void, unsigned> signal_item_selected;

private:
	ListData *data;
	bool own_data;
	DataObserver *observer;
	int sel_index;
	unsigned first;
	unsigned max_scroll;
	unsigned view_size;

	VSlider slider;
	std::vector<Item *> items;

public:
	List();
	List(ListData &);
private:
	void init();
public:
	virtual ~List();

	virtual const char *get_class() const { return "list"; }

	virtual void autosize();

	void set_data(ListData &);
	ListData &get_data() { return *data; }
	const ListData &get_data() const { return *data; }
private:
	void items_changed();
protected:
	virtual Item *create_item(unsigned);

public:
	void set_view_size(unsigned);
	void set_view_all();

	void set_selected_index(int);
	int get_selected_index() const { return sel_index; }

private:
	virtual void render_special(const Part &, GL::Renderer &) const;

public:
	virtual void button_press(int, int, unsigned);
private:
	virtual void on_geometry_change();
	virtual void on_style_change();

	void reposition_slider();
	void reposition_items();
	void check_view_range();
	void slider_value_changed(double);
};

} // namespace GLtk
} // namespace Msp

#endif
