#ifndef MSP_GLTK_TEXT_H_
#define MSP_GLTK_TEXT_H_

#include <string>
#include <vector>

namespace Msp {
namespace GLtk {

class Geometry;
class Part;
class PartCache;
class Style;

/**
Stores and renders text.  Supports multiline text.
*/
class Text
{
private:
	struct Line
	{
		unsigned start;
		unsigned length;
		unsigned width;
	};

	struct RenderData;
	struct CoordsToGeomData;

	const Style *style;
	std::string text;
	std::vector<Line> lines;

public:
	Text();
	Text(const Style &, const std::string &);

	void set_style(const Style *);

	unsigned get_width() const;
	unsigned get_height() const;

	void set(const std::string &);
	void erase(unsigned, unsigned);
	void insert(unsigned, const std::string &);
	const std::string &get() const { return text; }
	unsigned size() const { return text.size(); }
	unsigned get_n_lines() const { return lines.size(); }
	unsigned get_line_length(unsigned) const;
	void offset_to_coords(unsigned, unsigned &, unsigned &) const;
	unsigned coords_to_offset(unsigned, unsigned) const;
	Geometry coords_to_geometry(const Part &, const Geometry &, unsigned, unsigned, unsigned) const;

	void build(const Part &, const Geometry &, PartCache &) const;
	void build(const Part &, const Geometry &, unsigned, PartCache &) const;

	Text &operator=(const std::string &);
private:
	void find_lines();

	template<typename T, void (Text::*)(unsigned, const Geometry &, T &) const>
	void process_lines(const Part &, const Geometry &, unsigned, T &) const;

	void build_line(unsigned, const Geometry &, RenderData &) const;
	void coords_to_geom_line(unsigned, const Geometry &, CoordsToGeomData &) const;
};

} // namespace GLtk
} // namespace Msp

#endif
